import { QuickJSFFI, JSContextPointer, JSValuePointer, JSRuntimePointer, JSValueConstPointer } from './ffi';
import { LowLevelJavascriptVm, VmPropertyDescriptor, VmCallResult, VmFunctionImplementation } from './vm-interface';
declare const QuickJSModule: import("./quickjs-emscripten-module").QuickJSEmscriptenModule;
declare type CToHostCallbackFunctionImplementation = (ctx: JSContextPointer, this_ptr: JSValueConstPointer, argc: number, argv: JSValueConstPointer, fn_data_ptr: JSValueConstPointer) => JSValuePointer;
/**
 * A lifetime prevents access to a value after the lifetime has been
 * [[dispose]]ed.
 *
 * Typically, quickjs-emscripten uses Lifetimes to protect C memory pointers.
 */
export declare class Lifetime<T, Owner = never> {
    private readonly _value;
    private readonly disposer?;
    private readonly _owner?;
    private _alive;
    /**
     * When the Lifetime is disposed, it will call `disposer(_value)`. Use the
     * disposer function to implement whatever cleanup needs to happen at the end
     * of `value`'s lifetime.
     *
     * `_owner` is not used or controlled by the lifetime. It's just metadata for
     * the creator.
     */
    constructor(_value: T, disposer?: ((value: T) => void) | undefined, _owner?: Owner | undefined);
    get alive(): boolean;
    /**
     * The value this Lifetime protects. You must never retain the value - it
     * may become invalid, leading to memory errors.
     *
     * @throws If the lifetime has been [[dispose]]d already.
     */
    get value(): T;
    get owner(): Owner | undefined;
    /**
     * Dispose of [[value]] and perform cleanup.
     */
    dispose(): void;
    private assertAlive;
}
/**
 * A Lifetime that lives forever. Used for constants.
 */
export declare class StaticLifetime<T, Owner = never> extends Lifetime<T, Owner> {
    dispose(): void;
}
/**
 * QuickJSVm wraps a QuickJS Javascript runtime (JSRuntime*) and context (JSContext*).
 * This class's methods return {@link QuickJSHandle}, which wrap C pointers (JSValue*).
 * It's the caller's responsibility to call `.dispose()` on any
 * handles you create to free memory once you're done with the handle.
 *
 * Each QuickJSVm instance is isolated. You cannot share handles between different
 * QuickJSVm instances. You should create separate QuickJSVm instances for
 * untrusted code from different souces for isolation.
 *
 * Use [[QuickJS.createVm]] to create a new QuickJSVm.
 *
 * Create QuickJS values with methods like [[newNumber]], [[newString]], [[newObject]], and [[newFunction]].
 *
 * Call [[setProp]] or [[defineProp]] to customize objects. Use those methods with [[global]] to expose the
 * values you create to the interior of the interpreter, so they can be used in [[evalCode]].
 */
export declare class QuickJSVm implements LowLevelJavascriptVm<QuickJSHandle> {
    private readonly ctx;
    private readonly rt;
    private readonly module;
    private readonly ffi;
    private _undefined;
    private _global;
    private readonly lifetimes;
    /**
     * Use {@link QuickJS.createVm} to create a QuickJSVm instance.
     */
    constructor(args: {
        module: typeof QuickJSModule;
        ffi: QuickJSFFI;
        ctx: Lifetime<JSContextPointer>;
        rt: Lifetime<JSRuntimePointer>;
    });
    /**
     * [`undefined`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/undefined).
     */
    get undefined(): QuickJSHandle;
    /**
     * [`global`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects).
     * A handle to the global object inside the interpreter.
     * You can set properties to create global variables.
     */
    get global(): QuickJSHandle;
    /**
     * `typeof` operator. **Not** [standards compliant](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/typeof).
     *
     * @remarks
     * Does not support BigInt values correctly.
     */
    typeof(handle: QuickJSHandle): string;
    /**
     * Converts a Javascript number into a QuckJS value.
     */
    newNumber(num: number): QuickJSHandle;
    /**
     * Converts `handle` into a Javascript number.
     * @returns `NaN` on error, othewise a `number`.
     */
    getNumber(handle: QuickJSHandle): number;
    /**
     * Create a QuickJS [string](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String) value.
     */
    newString(str: string): QuickJSHandle;
    /**
     * Converts `handle` to a Javascript string.
     */
    getString(handle: QuickJSHandle): string;
    /**
     * `{}`.
     * Create a new QuickJS [object](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Object_initializer).
     *
     * @param prototype - Like [`Object.create`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/create).
     */
    newObject(prototype?: QuickJSHandle): QuickJSHandle;
    /**
     * Convert a Javascript function into a QuickJS function value.
     * See [[VmFunctionImplementation]] for more details.
     *
     * A [[VmFunctionImplementation]] should not free its arguments or its retun
     * value. A VmFunctionImplementation should also not retain any references to
     * its veturn value.
     */
    newFunction(name: string, fn: VmFunctionImplementation<QuickJSHandle>): QuickJSHandle;
    /**
     * `handle[key]`.
     * Get a property from a JSValue.
     *
     * @param key - The property may be specified as a JSValue handle, or as a
     * Javascript string (which will be converted automatically).
     */
    getProp(handle: QuickJSHandle, key: string | QuickJSHandle): QuickJSHandle;
    /**
     * `handle[key] = value`.
     * Set a property on a JSValue.
     *
     * @remarks
     * Note that the QuickJS authors recommend using [[defineProp]] to define new
     * properties.
     *
     * @param key - The property may be specified as a JSValue handle, or as a
     * Javascript string (which will be converted automatically).
     */
    setProp(handle: QuickJSHandle, key: string | QuickJSHandle, value: QuickJSHandle): void;
    /**
     * [`Object.defineProperty(handle, key, descriptor)`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Object/defineProperty).
     *
     * @param key - The property may be specified as a JSValue handle, or as a
     * Javascript string (which will be converted automatically).
     */
    defineProp(handle: QuickJSHandle, key: string | QuickJSHandle, descriptor: VmPropertyDescriptor<QuickJSHandle>): void;
    /**
     * [`func.call(thisVal, ...args)`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Function/call).
     * Call a JSValue as a function.
     *
     * See [[unwrapResult]], which will throw if the function returned an error, or
     * return the result handle directly.
     *
     * @returns A result. If the function threw, result `error` be a handle to the exception.
     */
    callFunction(func: QuickJSHandle, thisVal: QuickJSHandle, ...args: QuickJSHandle[]): VmCallResult<QuickJSHandle>;
    /**
     * Like [`eval(code)`](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/eval#Description).
     * Evauatetes the Javascript source `code` in the global scope of this VM.
     *
     * See [[unwrapResult]], which will throw if the function returned an error, or
     * return the result handle directly.
     *
     * *Note: this does not protect against infinite loops*.
     *
     * @returns The last statement's value. If the code threw, result `error` be
     * a handle to the exception.
     */
    evalCode(code: string): VmCallResult<QuickJSHandle>;
    /**
     * Dump a JSValue to Javascript in a best-effort fashion.
     * Returns `handle.toString()` if it cannot be serialized to JSON.
     */
    dump(handle: QuickJSHandle): any;
    /**
     * Unwrap a VmCallResult, returning it's value on success, and throwing the dumped
     * error on failure.
     */
    unwrapResult(result: VmCallResult<QuickJSHandle>): QuickJSHandle;
    /**
     * Dispose of this VM's underlying resources.
     *
     * @throws If Calling this method without disposing of all created handles
     * will result in an error.
     */
    dispose(): void;
    private fnNextId;
    private fnMap;
    /**
     * @hidden
     */
    cToHostCallbackFunction: CToHostCallbackFunctionImplementation;
    private assertOwned;
    private freeJSValue;
    private heapValueHandle;
    private toPointerArray;
    private errorToHandle;
}
/**
 * A QuickJSHandle to a constant that will never change, and does not need to
 * be disposed.
 */
export declare type StaticJSValue = StaticLifetime<JSValueConstPointer>;
/**
 * A QuickJSHandle to a borrowed value that does not need to be disposed.
 *
 * In QuickJS, a JSValueConst is a "borrowed" reference that isn't owned by the
 * current scope. That means that the current scope should not `JS_FreeValue`
 * it, or retain a reference to it after the scope exits, because it may be
 * freed by its owner.
 *
 * quickjs-emscripten takes care of disposing JSValueConst references.
 */
export declare type JSValueConst = Lifetime<JSValueConstPointer, QuickJSVm>;
/**
 * A owned QuickJSHandle that should be disposed or returned.
 *
 * The QuickJS interpreter passes Javascript values between functions as
 * `JSValue` structs that references some internal data. Because passing
 * structs cross the Empscripten FFI interfaces is bothersome, we use pointers
 * to these structs instead.
 *
 * A JSValue reference is "owned" in its scope. before exiting the scope, it
 * should be freed,  by calling `JS_FreeValue(ctx, js_value)`) or returned from
 * the scope. We extend that contract - a JSValuePointer (`JSValue*`) must also
 * be `free`d.
 *
 * You can do so from Javascript by calling the .dispose() method.
 */
export declare type JSValue = Lifetime<JSValuePointer, QuickJSVm>;
/**
 * Wraps a C pointer to a QuickJS JSValue, which represents a Javascript value inside
 * a QuickJS virtual machine.
 *
 * Values must not be shared between QuickJSVm instances.
 * You must dispose of any handles you create by calling the `.dispose()` method.
 */
export declare type QuickJSHandle = StaticJSValue | JSValue | JSValueConst;
/**
 * QuickJS presents a Javascript interface to QuickJS, a Javascript interpreter that
 * supports ES2019.
 *
 * QuickJS is a singleton. Use the [[getQuickJS]] function to instantiate
 * or retrieve an instance.
 *
 * Use the {@link QuickJS.createVm} method to create a {@link QuickJSVm}.
 *
 * Use the {@link QuickJS.evalCode} method as a shortcut evaluate Javascript safely
 * and return the result as a native Javascript value.
 */
export declare class QuickJS {
    private ffi;
    private vmMap;
    private module;
    constructor();
    /**
     * Create a QuickJS VM.
     *
     * Each VM is completely independent - you cannot share handles between
     * VMs.
     */
    createVm(): QuickJSVm;
    /**
     * One-off evaluate code without needing to create a VM.
     * The result is coerced to a native Javascript value using JSON
     * serialization, so values unsupported by JSON will be dropped.
     *
     * If you need more control over how the code executes, create a
     * [[QuickJSVm]] instance and use its [[QuickJSVm.evalCode]] method.
     *
     * *Note: this does not protect against infinite loops.*
     *
     * @throws If `code` throws during evaluation, the exception will be
     * converted into a Javascript value and throw.
     */
    evalCode(code: string): unknown;
    private cToHostCallbackFunction;
}
/**
 * This is the top-level entrypoint for the quickjs-emscripten library.
 * Get the root QuickJS API.
 */
export declare function getQuickJS(): Promise<QuickJS>;
export {};
